const fs = require('fs');

const amrnb_frame_sizes = [12, 13, 15, 17, 19, 20, 26, 31,  5, 0, 0, 0, 0, 0, 0, 0];
const amrwb_frame_sizes = [17, 23, 32, 36, 40, 46, 50, 58, 60, 5, 0, 0, 0, 0, 0, 0];

function getAMRDuration(amrData) {

  const AMRFLAG = '#!AMR\n';
  const AMRWBFLAG = '#!AMR-WB\n';
  const flag = amrData.toString('ascii', 0, 6);
  const wbflag = amrData.toString('ascii', 0, 9);

  let amrType = 0;
  if (flag === AMRFLAG) amrType = 1;
  else if (wbflag === AMRWBFLAG) amrType = 2;

  if (amrType === 0) return 0;

  let nbFrames = 0;
  let pos = (amrType === 1) ? 6 : 9;
  while( pos < amrData.length) {
    const frameType = (amrData[pos] & 0x78) >> 3;
    const frameSize = (amrType === 1) ? amrnb_frame_sizes[frameType] : amrwb_frame_sizes[frameType];
    pos += (1 + frameSize);
    nbFrames ++;
  }

  return nbFrames * 20;

}

const data = fs.readFileSync('../wav/voice.amr');
const duration = getAMRDuration(data);
console.log(duration + 'ms');

